@@ -366,7 +366,8 @@ rmt_open__ (const char *file_name, int open_mode, int bias,
   char *remote_host;		/* remote host name */
   char *remote_file;		/* remote file name (often a device) */
   char *remote_user;		/* remote user name */
-
+  int rc;
+  
   /* Find an unused pair of file descriptors.  */
 
   for (remote_pipe_number = 0;
@@ -425,7 +426,22 @@ rmt_open__ (const char *file_name, int open_mode, int bias,
   }
 
   /* FIXME: Should somewhat validate the decoding, here.  */
+#if HAVE_GETADDRINFO
+  {
+    struct addrinfo *res;
+    rc = getaddrinfo (remote_host, NULL, NULL, &res);
+    if (rc == 0)
+      freeaddrinfo (res);
+  }
+#else
+  rc = gethostbyname (remote_host) == NULL;
+#endif
+  
+  if (rc)
+    error (EXIT_ON_EXEC_ERROR, 0,
+	   _("Cannot resolve hostname %s"), remote_host);
 
+  
   if (remote_user && *remote_user == '\0')
     remote_user = 0;
 
@@ -461,7 +477,7 @@ rmt_open__ (const char *file_name, int open_mode, int bias,
 	return -1;
 #endif
       }
-    remote_shell_basename = base_name (remote_shell);
+    remote_shell_basename = last_component (remote_shell);
 
     /* Set up the pipes for the `rsh' command, and fork.  */
 
