@@ -258,7 +258,7 @@ create_defered_links (struct cpio_file_stat *file_hdr)
 {
   struct deferment *d;
   struct deferment *d_prev;
-  int	ino;
+  ino_t	ino;
   int 	maj;
   int   min;
   int 	link_res;
@@ -306,7 +306,7 @@ create_defered_links_to_skipped (struct cpio_file_stat *file_hdr,
 {
   struct deferment *d;
   struct deferment *d_prev;
-  int	ino;
+  ino_t	ino;
   int 	maj;
   int   min;
   if (file_hdr->c_filesize == 0)
@@ -566,6 +566,30 @@ copyin_regular_file (struct cpio_file_stat* file_hdr, int in_file_des)
     }
 }
 
+static int
+copyin_mkdir (struct cpio_file_stat *file_hdr, int *setstat_delayed)
+{
+  int rc;
+  mode_t mode = file_hdr->c_mode;
+  if (!(file_hdr->c_mode & S_IWUSR))
+    {
+      rc = mkdir (file_hdr->c_name, mode | S_IWUSR);
+      if (rc == 0)
+	{
+	  struct stat st;
+	  cpio_to_stat (&st, file_hdr);
+	  delay_set_stat (file_hdr->c_name, &st, false);
+	  *setstat_delayed = 1;
+	}
+    }
+  else
+    {
+      rc = mkdir (file_hdr->c_name, mode);
+      *setstat_delayed = 0;
+    }
+  return rc;
+}
+
 static void
 copyin_directory (struct cpio_file_stat *file_hdr, int existing_dir)
 {
@@ -574,7 +598,8 @@ copyin_directory (struct cpio_file_stat *file_hdr, int existing_dir)
   int cdf_flag;                 /* True if file is a CDF.  */
   int cdf_char;                 /* Index of `+' char indicating a CDF.  */
 #endif
-
+  int setstat_delayed = 0;
+  
   if (to_stdout_option)
     return;
   
@@ -610,14 +635,14 @@ copyin_directory (struct cpio_file_stat *file_hdr, int existing_dir)
 	  cdf_flag = 1;
 	}
 #endif
-      res = mkdir (file_hdr->c_name, file_hdr->c_mode);
+      res = copyin_mkdir (file_hdr, &setstat_delayed);
     }
   else
     res = 0;
   if (res < 0 && create_dir_flag)
     {
       create_all_directories (file_hdr->c_name);
-      res = mkdir (file_hdr->c_name, file_hdr->c_mode);
+      res = copyin_mkdir (file_hdr, &setstat_delayed);
     }
   if (res < 0)
     {
@@ -645,7 +670,8 @@ copyin_directory (struct cpio_file_stat *file_hdr, int existing_dir)
 	}
     }
 
-  set_perms (-1, file_hdr); 
+  if (!setstat_delayed && repair_delayed_set_stat (file_hdr) == 0)
+    set_perms (-1, file_hdr);
 }
 
 static void
@@ -1434,8 +1460,8 @@ process_copy_in ()
 	  struct cpio_file_stat *h;
 	  h = &file_hdr;
 	  fprintf (stderr, 
-		"magic = 0%o, ino = %d, mode = 0%o, uid = %d, gid = %d\n",
-		h->c_magic, h->c_ino, h->c_mode, h->c_uid, h->c_gid);
+		"magic = 0%o, ino = %ld, mode = 0%o, uid = %d, gid = %d\n",
+		h->c_magic, (long)h->c_ino, h->c_mode, h->c_uid, h->c_gid);
 	  fprintf (stderr, 
 		"nlink = %d, mtime = %d, filesize = %d, dev_maj = 0x%x\n",
 		h->c_nlink, h->c_mtime, h->c_filesize, h->c_dev_maj);
